SUBROUTINE orthogonal_cholesky(A,n_row,n_col,ng_n,U)
  ! On entry, the matrix A (of size n_row X n_col) contains column vectors a1, a2, a3, ... an_col, that are not orthogonal each other.
  ! however, only 1:ng_n of each column 1:n_col is used as the vector
  ! On exit, A contains column vectors that are orthognormal.

  ! Decompose A = (Psi**H * Psi) = U**H * U.
  ! The orthogonal vectors Phi = Psi * U**(-1).
  ! This is equivalent to Gram-Schmidt orthogonalization progressing from the first column vector of Psi.

  USE fft_data
  USE load_data
  USE DATA
  
  IMPLICIT NONE

  INCLUDE 'mpif.h'
  INCLUDE 'param.escan_real_f90'
  
  INTEGER, INTENT(in) :: n_row, n_col
  COMPLEX*16, DIMENSION(n_row,n_col) :: A
  COMPLEX*16, ALLOCATABLE,DIMENSION(:,:) :: CC
  
  COMPLEX*16, DIMENSION(n_col,n_col) :: S, U
  INTEGER :: i, j, k, ierr, info
  
  CHARACTER(len=1) :: uplo, trans
  COMPLEX*16 :: one, zero


  complex*16 :: sss
  integer :: ii, mk


  !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
  ! Calculate overlap matrix.
  ! S = A* . A
  !  uplo = 'l'
  uplo = 'u'
  trans = 'c'
  one = DCMPLX(1.d0,0.d0)
  zero = DCMPLX(0.d0,0.d0)

  ! important
  S = DCMPLX(0.d0,0.d0)    

!  CALL zherk(uplo,trans,n_col,ng_n,one,A,n_row,zero,S,n_col)

  CALL zherk(uplo,trans,n_col,ng_n,vol,A,n_row,zero,S,n_col)

! it will be much better if we have a compact form for S, so we don't need to wast half of the space for mpi_allreduce

  CALL mpi_allreduce(S,U,n_col*n_col,MPI_DOUBLE_COMPLEX,MPI_SUM,MPI_COMM_K,ierr)

! the lower triangle of S and U is zero

  !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
  ! Cholesky decomposition
  ! A = U**H * U
  uplo = 'U'
  
  CALL zpotrf(uplo,n_col,U,n_col,info)

  IF (info /= 0) THEN
     WRITE(6,*) "Somthing is wrong with cholesky decomposition of S. info", info
     STOP
  ENDIF

! the lower triangle of U is still zero, never used


     

  !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
  ! Calculate the inverse of cholesky factor.
  ! U = U**(-1)


  CALL ztrtri(uplo,'N',n_col,U,n_col,info)

  IF (info /= 0) THEN
     WRITE(6,*) "Somthing is wrong with inversion of cholesky factor of U. info", info
     STOP
  ENDIF

! the lower triangle of U is still zero, never used

  ! Throw away the part that are not referenced in zpotrf.
  ! this is needed for ztrtr, a bit strange, should not be needed 
  ! DO i=1, n_col-1
  !   U(i+1:n_col,i) = zero
  ! ENDDO 


  !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
  ! Orthogonalize A.
  ! A = A * U**(-1)
  ! A = MATMUL(A,U)  
  ! unfortunately, half is wasted, because half of U is zero. 

  allocate(CC(n_row,n_col))

!  CALL zgemm('N','N',ng_n,n_col,n_col,one,A,n_row,U,n_col,zero,A,n_row)

  CALL zgemm('N','N',ng_n,n_col,n_col,one,A,n_row,U,n_col,zero,CC,n_row)

   A(1:ng_n,1:n_col)=CC(1:ng_n,1:n_col)

   deallocate(CC)
  

END SUBROUTINE orthogonal_cholesky
