SUBROUTINE orthogonal_projection(A,row_a,col_a,B,row_b,col_b,n_mul)
  ! Orthogonalize column vectors a_i in matrice A to column vectors b_j in matrice B for all j .le. i.
  ! On entry, the matrice A contains complex*16 column vectors that are not orthogonal to B.
  ! On exit, the column vectors a_i are orthogonal to b_j (j .le. i).
  ! The vectors a_i are not normalized.
  !
  ! Things to test.
  ! 1. Ignoring of projections to higher wavefunctions is necessary? 
  !....... Makes little difference.
  !....... See all_band/GaAs64at_3.6_scf (Using the projection to all states.)
  ! 2. If so, we waste half of the computation by using level 3 BLAS.
  !    We might want to use level 2 BLAS.

  USE fft_data
  USE load_data
  USE DATA
  
  IMPLICIT NONE

  INCLUDE 'mpif.h'
  INCLUDE 'param.escan_real_f90'
  
  INTEGER, INTENT(in) :: row_a, col_a, row_b, col_b, n_mul
  DOUBLE COMPLEX, DIMENSION(row_a,col_a) :: A
  DOUBLE COMPLEX, DIMENSION(row_b,col_b), INTENT(in) :: B
  
  DOUBLE COMPLEX, DIMENSION(col_b,col_a) :: S, SS
  DOUBLE COMPLEX, PARAMETER :: one=(1.d0,0.d0), minus_one=(-1.d0,0.d0), zero=(0.d0,0.d0)
  INTEGER :: ierr, i, irow, icol


  !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
  ! Calculate overlap matrix.
  ! S = B**H * A
  ! S_ij is the projection of vector a_i to vector b_j.
  CALL zgemm('c','n',col_b,col_a,n_mul,one,B,row_b,A,row_a,zero,S,col_b)
  
  CALL mpi_allreduce(S,SS,col_a*col_b,MPI_DOUBLE_COMPLEX,MPI_SUM,MPI_COMM_K,ierr)


  !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
  ! Ignore projection to higher wavefunctions.
  ! Test if this is necessary.
  DO icol=1, col_a
     DO irow=1, icol
        S(irow,icol) = SS(irow,icol)*vol
     ENDDO

     DO irow=icol+1, col_b
        S(irow,icol) = zero
     ENDDO
  ENDDO
!!$  S = SS*vol


  !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
  ! Orthogonalize A.
  ! A = A - B * S
  CALL zgemm('n','n',n_mul,col_a,col_b,minus_one,B,row_b,S,col_b,one,A,row_a)

END SUBROUTINE orthogonal_projection
