SUBROUTINE orthogonal_cholesky2(A,n_row,n_col,ng_n)
  ! On entry, the matrix A (of size n_row X n_col) contains column vectors a1, a2, a3, ... an_col, that are not orthogonal each other.
  ! however, only 1:ng_n of each column 1:n_col is used as the vector
  ! On exit, A contains column vectors that are orthognormal.

  ! Decompose A = (Psi**H * Psi) = U**H * U.
  ! The orthogonal vectors Phi = Psi * U**(-1).
  ! This is equivalent to Gram-Schmidt orthogonalization progressing from the first column vector of Psi.

  USE fft_data
  USE load_data
  USE DATA
  
  IMPLICIT NONE

  INCLUDE 'mpif.h'
  INCLUDE 'param.escan_real_f90'
  
  INTEGER, INTENT(in) :: n_row, n_col
  COMPLEX*16, DIMENSION(n_row,n_col) :: A
  COMPLEX*16, ALLOCATABLE,DIMENSION(:,:) :: CC
  
  COMPLEX*16, DIMENSION(n_col,n_col) :: S, U
  INTEGER :: i, j, k, ierr, info
  
  CHARACTER(len=1) :: uplo, trans
  COMPLEX*16 :: one, zero


  complex*16 :: sss
  integer :: ii, mk


  !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
  ! Calculate overlap matrix.
  ! S = A* . A
  !  uplo = 'l'
  uplo = 'u'
  trans = 'c'
  one = DCMPLX(1.d0,0.d0)
  zero = DCMPLX(0.d0,0.d0)

  ! important
  S = DCMPLX(0.d0,0.d0)    

  CALL zherk(uplo,trans,n_col,ng_n,vol,A,n_row,zero,S,n_col)

  CALL mpi_allreduce(S,U,n_col*n_col,MPI_DOUBLE_COMPLEX,MPI_SUM,MPI_COMM_K,ierr)

  ! normalization of the overlap matrice.
  DO i=1, n_col
     DO j=1, n_col
        S(i,j) = U(i,j)/SQRT(U(i,i)*U(j,j))
     ENDDO
  ENDDO


  !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
  ! Cholesky decomposition
  ! A = S**H * S
  uplo = 'U'
  
  CALL zpotrf(uplo,n_col,S,n_col,info)

  IF (info /= 0) THEN
     WRITE(6,*) "Somthing is wrong with cholesky2 decomposition of S. info", info
     STOP
  ENDIF


  !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
  ! Calculate the inverse of cholesky factor.
  ! U = U**(-1)

  CALL ztrtri(uplo,'N',n_col,S,n_col,info)

  IF (info /= 0) THEN
     WRITE(6,*) "Somthing is wrong with inversion of cholesky2 factor of U. info", info
     STOP
  ENDIF


  !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
  ! Orthogonalize A.
  ! A = A * S**(-1)
  ! A = MATMUL(A,S)  
  ! unfortunately, half is wasted, because half of U is zero. 

  allocate(CC(n_row,n_col))

  CALL zgemm('N','N',ng_n,n_col,n_col,one,A,n_row,S,n_col,zero,CC,n_row)

  DO i=1, n_col
     A(1:ng_n,i)=CC(1:ng_n,i)/SQRT(U(i,i))
  ENDDO

  DEALLOCATE(CC)
  

 END SUBROUTINE orthogonal_cholesky2
